@@ -5,15 +5,17 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
-#include "chunk_list.h"
-#include <cstring>
-#include <cstdlib>
 
-#include "uncrustify_types.h"
+#include "chunk_list.h"
+
 #include "ListManager.h"
 #include "prototypes.h"
-#include "uncrustify.h"
 #include "space.h"
+#include "uncrustify.h"
+#include "uncrustify_types.h"
+
+#include <cstdlib>
+#include <cstring>
 
 
 typedef ListManager<chunk_t> ChunkList_t;
@@ -77,6 +79,27 @@ typedef chunk_t * (*search_t)(chunk_t *cur, scope_e scope);
 static chunk_t *chunk_search(chunk_t *cur, const check_t check_fct, const scope_e scope = scope_e::ALL, const direction_e dir = direction_e::FORWARD, const bool cond = true);
 
 
+/**
+ * @brief search for a chunk that satisfies a condition in a chunk list.
+ *
+ * This function is similar to chunk_search, except that it is tweaked to
+ * handle searches inside of preprocessor directives. Specifically, if the
+ * starting token is inside a preprocessor directive, it will ignore a line
+ * continuation, and will abort the search if it reaches the end of the
+ * directive. This function only searches forward.
+ *
+ * @param  cur        chunk to start search at
+ * @param  check_fct  compare function
+ * @param  scope      code parts to consider for search
+ * @param  cond       success condition
+ *
+ * @retval nullptr  no requested chunk was found or invalid parameters provided
+ * @retval chunk_t  pointer to the found chunk or pointer to the chunk at the
+ *                  end of the preprocessor directive
+ */
+static chunk_t *chunk_ppa_search(chunk_t *cur, const check_t check_fct, const bool cond = true);
+
+
 static void chunk_log(chunk_t *pc, const char *text);
 
 
@@ -140,9 +163,6 @@ static chunk_t *chunk_search_typelevel(chunk_t *cur, c_token_t type, scope_e sco
 static chunk_t *chunk_get_ncnlnp(chunk_t *cur, const scope_e scope = scope_e::ALL, const direction_e dir = direction_e::FORWARD);
 
 
-static chunk_t *chunk_get_ncnlnpnd(chunk_t *cur, const scope_e scope = scope_e::ALL, const direction_e dir = direction_e::FORWARD);
-
-
 /**
  * @brief searches a chunk that holds a specific string
  *
@@ -223,6 +243,31 @@ chunk_t *chunk_search_next_cat(chunk_t *pc, const c_token_t cat)
 }
 
 
+bool are_chunks_in_same_line(chunk_t *start, chunk_t *end)
+{
+   chunk_t *tmp;
+
+   if (start != nullptr)
+   {
+      tmp = chunk_get_next(start);
+   }
+   else
+   {
+      return(false);
+   }
+
+   while (tmp != nullptr && tmp != end)
+   {
+      if (chunk_is_token(tmp, CT_NEWLINE))
+      {
+         return(false);
+      }
+      tmp = chunk_get_next(tmp);
+   }
+   return(true);
+}
+
+
 static chunk_t *chunk_search_type(chunk_t *cur, const c_token_t type,
                                   const scope_e scope, const direction_e dir)
 {
@@ -238,6 +283,7 @@ static chunk_t *chunk_search_type(chunk_t *cur, const c_token_t type,
       pc = search_function(pc, scope); // in either direction while
    } while (  pc != nullptr            // the end of the list was not reached yet
            && pc->type != type);       // and the demanded chunk was not found either
+
    return(pc);                         // the latest chunk is the searched one
 }
 
@@ -254,24 +300,10 @@ static chunk_t *chunk_search_typelevel(chunk_t *cur, c_token_t type, scope_e sco
    do                                  // loop over the chunk list
    {
       pc = search_function(pc, scope); // in either direction while
-#if DEBUG
-      if (pc != nullptr)
-      {
-         if (pc->type == CT_NEWLINE)
-         {
-            LOG_FMT(LCHUNK, "%s(%d): orig_line is %zu, orig_col is %zu, NEWLINE\n",
-                    __func__, __LINE__, pc->orig_line, pc->orig_col);
-         }
-         else
-         {
-            LOG_FMT(LCHUNK, "%s(%d): orig_line is %zu, orig_col is %zu, pc->text() '%s', type is %s\n",
-                    __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-         }
-      }
-#endif
-   } while (  pc != nullptr        // the end of the list was not reached yet
+   } while (  pc != nullptr            // the end of the list was not reached yet
            && (is_expected_type_and_level(pc, type, level) == false));
-   return(pc);                     // the latest chunk is the searched one
+
+   return(pc);                         // the latest chunk is the searched one
 }
 
 
@@ -288,6 +320,7 @@ static chunk_t *chunk_search_str(chunk_t *cur, const char *str, size_t len, scop
       pc = search_function(pc, scope); // in either direction while
    } while (  pc != nullptr            // the end of the list was not reached yet
            && (is_expected_string_and_level(pc, str, level, len) == false));
+
    return(pc);                         // the latest chunk is the searched one
 }
 
@@ -306,10 +339,49 @@ static chunk_t *chunk_search(chunk_t *cur, const check_t check_fct, const scope_
       pc = search_function(pc, scope);  // in either direction while
    } while (  pc != nullptr             // the end of the list was not reached yet
            && (check_fct(pc) != cond)); // and the demanded chunk was not found either
+
    return(pc);                          // the latest chunk is the searched one
 }
 
 
+static chunk_t *chunk_ppa_search(chunk_t *cur, const check_t check_fct, const bool cond)
+{
+   if (cur && !cur->flags.test(PCF_IN_PREPROC))
+   {
+      // if not in preprocessor, do a regular search
+      return(chunk_search(cur, check_fct, scope_e::ALL,
+                          direction_e::FORWARD, cond));
+   }
+   chunk_t *pc = cur;
+
+   while (pc != nullptr && (pc = pc->next) != nullptr)
+   {
+      if (!pc->flags.test(PCF_IN_PREPROC))
+      {
+         // Bail if we run off the end of the preprocessor directive, but
+         // return the next token, NOT nullptr, because the caller may need to
+         // know where the search ended
+         assert(chunk_is_token(pc, CT_NEWLINE));
+         return(pc);
+      }
+
+      if (chunk_is_token(pc, CT_NL_CONT))
+      {
+         // Skip line continuation
+         continue;
+      }
+
+      if (check_fct(pc) == cond)
+      {
+         // Requested token was found
+         return(pc);
+      }
+   }
+   // Ran out of tokens
+   return(nullptr);
+}
+
+
 /* @todo maybe it is better to combine chunk_get_next and chunk_get_prev
  * into a common function However this should be done with the preprocessor
  * to avoid addition check conditions that would be evaluated in the
@@ -321,21 +393,24 @@ chunk_t *chunk_get_next(chunk_t *cur, scope_e scope)
       return(nullptr);
    }
    chunk_t *pc = g_cl.GetNext(cur);
+
    if (pc == nullptr || scope == scope_e::ALL)
    {
       return(pc);
    }
-   if (cur->flags & PCF_IN_PREPROC)
+
+   if (cur->flags.test(PCF_IN_PREPROC))
    {
       // If in a preproc, return nullptr if trying to leave
-      if ((pc->flags & PCF_IN_PREPROC) == 0)
+      if (!pc->flags.test(PCF_IN_PREPROC))
       {
          return(nullptr);
       }
       return(pc);
    }
+
    // Not in a preproc, skip any preproc
-   while (pc != nullptr && (pc->flags & PCF_IN_PREPROC))
+   while (pc != nullptr && pc->flags.test(PCF_IN_PREPROC))
    {
       pc = g_cl.GetNext(pc);
    }
@@ -350,21 +425,24 @@ chunk_t *chunk_get_prev(chunk_t *cur, scope_e scope)
       return(nullptr);
    }
    chunk_t *pc = g_cl.GetPrev(cur);
+
    if (pc == nullptr || scope == scope_e::ALL)
    {
       return(pc);
    }
-   if (cur->flags & PCF_IN_PREPROC)
+
+   if (cur->flags.test(PCF_IN_PREPROC))
    {
       // If in a preproc, return NULL if trying to leave
-      if ((pc->flags & PCF_IN_PREPROC) == 0)
+      if (!pc->flags.test(PCF_IN_PREPROC))
       {
          return(nullptr);
       }
       return(pc);
    }
+
    // Not in a preproc, skip any preproc
-   while (pc != nullptr && (pc->flags & PCF_IN_PREPROC))
+   while (pc != nullptr && pc->flags.test(PCF_IN_PREPROC))
    {
       pc = g_cl.GetPrev(pc);
    }
@@ -380,9 +458,10 @@ chunk_t *chunk_dup(const chunk_t *pc_in)
    {
       // @todo clean up properly before crashing
       LOG_FMT(LERR, "Failed to allocate memory\n");
+      log_func_stack_inline(LSETFLG);
+      log_flush(true);
       exit(EXIT_FAILURE);
    }
-
    // Copy all fields and then init the entry
    *pc = *pc_in; // TODO: what happens if pc_in == nullptr?
    g_cl.InitEntry(pc);
@@ -393,9 +472,25 @@ chunk_t *chunk_dup(const chunk_t *pc_in)
 
 static void chunk_log_msg(chunk_t *chunk, const log_sev_t log, const char *str)
 {
-   LOG_FMT(log, "%s %zu:%zu '%s' [%s]",
-           str, chunk->orig_line, chunk->orig_col, chunk->text(),
-           get_token_name(chunk->type));
+   LOG_FMT(log, "%s orig_line is %zu, orig_col is %zu, ",
+           str, chunk->orig_line, chunk->orig_col);
+
+   if (chunk_is_token(chunk, CT_NEWLINE))
+   {
+      LOG_FMT(log, "<Newline>,");
+   }
+   else if (chunk_is_token(chunk, CT_VBRACE_OPEN))
+   {
+      LOG_FMT(log, "<VBRACE_OPEN>,");
+   }
+   else if (chunk_is_token(chunk, CT_VBRACE_CLOSE))
+   {
+      LOG_FMT(log, "<VBRACE_CLOSE>,");
+   }
+   else
+   {
+      LOG_FMT(log, "text() is '%s', type is %s,\n", chunk->text(), get_token_name(chunk->type));
+   }
 }
 
 
@@ -411,20 +506,21 @@ static void chunk_log(chunk_t *pc, const char *text)
 
       chunk_log_msg(pc, log, text);
 
-      if (prev && next)
+      if (prev != nullptr && next != nullptr)
       {
          chunk_log_msg(prev, log, " @ between");
          chunk_log_msg(next, log, " and");
       }
-      else if (next)
+      else if (next != nullptr)
       {
          chunk_log_msg(next, log, " @ before");
       }
-      else if (prev)
+      else if (prev != nullptr)
       {
          chunk_log_msg(prev, log, " @ after");
       }
-      LOG_FMT(log, " stage=%d", (int)cpd.unc_stage);
+      LOG_FMT(log, " stage is %s",
+              get_unc_stage_name(cpd.unc_stage));
       log_func_stack_inline(log);
    }
 }
@@ -444,7 +540,7 @@ chunk_t *chunk_add_before(const chunk_t *pc_in, chunk_t *ref)
 
 void chunk_del(chunk_t *pc)
 {
-   chunk_log(pc, "chunk_del");
+   chunk_log(pc, "chunk_del(B):");
    g_cl.Pop(pc);
    delete pc;
 }
@@ -458,7 +554,7 @@ void chunk_move_after(chunk_t *pc_in, chunk_t *ref)
 
    // HACK: Adjust the original column
    pc_in->column       = ref->column + space_col_align(ref, pc_in);
-   pc_in->orig_col     = static_cast<UINT32>(pc_in->column);
+   pc_in->orig_col     = pc_in->column;
    pc_in->orig_col_end = pc_in->orig_col + pc_in->len();
 }
 
@@ -499,15 +595,15 @@ chunk_t *chunk_get_next_ncnlnp(chunk_t *cur, scope_e scope)
 }
 
 
-chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur, scope_e scope)
+chunk_t *chunk_ppa_get_next_ncnl(chunk_t *cur)
 {
-   return(chunk_get_ncnlnp(cur, scope, direction_e::BACKWARD));
+   return(chunk_ppa_search(cur, chunk_is_comment_or_newline, false));
 }
 
 
-chunk_t *chunk_get_prev_ncnlnpnd(chunk_t *cur, scope_e scope)
+chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur, scope_e scope)
 {
-   return(chunk_get_ncnlnpnd(cur, scope, direction_e::BACKWARD));
+   return(chunk_get_ncnlnp(cur, scope, direction_e::BACKWARD));
 }
 
 
@@ -541,6 +637,12 @@ chunk_t *chunk_get_prev_ncnl(chunk_t *cur, scope_e scope)
 }
 
 
+chunk_t *chunk_get_prev_ncnlni(chunk_t *cur, scope_e scope)
+{
+   return(chunk_search(cur, chunk_is_comment_or_newline_or_ignored, scope, direction_e::BACKWARD, false));
+}
+
+
 chunk_t *chunk_get_prev_nc(chunk_t *cur, scope_e scope)
 {
    return(chunk_search(cur, chunk_is_comment, scope, direction_e::BACKWARD, false));
@@ -580,6 +682,7 @@ bool chunk_is_newline_between(chunk_t *start, chunk_t *end)
          return(true);
       }
    }
+
    return(false);
 }
 
@@ -599,7 +702,6 @@ chunk_t *chunk_first_on_line(chunk_t *pc)
    {
       first = pc;
    }
-
    return(first);
 }
 
@@ -613,14 +715,13 @@ bool chunk_is_last_on_line(chunk_t &pc)  //TODO: pc should be const here
    {
       return(true);
    }
-
    // if the next chunk is a newline then pc is the last chunk on its line
    const auto *next = chunk_get_next(&pc);
-   if (next != nullptr && next->type == CT_NEWLINE)
+
+   if (chunk_is_token(next, CT_NEWLINE))
    {
       return(true);
    }
-
    return(false);
 }
 
@@ -638,7 +739,6 @@ void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
    {
       return;
    }
-
    /*
     * Example start:
     * ? - start1 - a1 - b1 - nl1 - ? - ref2 - start2 - a2 - b2 - nl2 - ?
@@ -654,7 +754,6 @@ void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
       g_cl.AddBefore(pc2, pc1);
       pc2 = tmp;
    }
-
    /*
     * Should now be:
     * ? - start2 - a2 - b2 - start1 - a1 - b1 - nl1 - ? - ref2 - nl2 - ?
@@ -666,6 +765,7 @@ void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
    {
       chunk_t *tmp = chunk_get_next(pc1);
       g_cl.Pop(pc1);
+
       if (ref2 != nullptr)
       {
          g_cl.AddAfter(pc1, ref2);
@@ -677,7 +777,6 @@ void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
       ref2 = pc1;
       pc1  = tmp;
    }
-
    /*
     * Should now be:
     * ? - start2 - a2 - b2 - nl1 - ? - ref2 - start1 - a1 - b1 - nl2 - ?
@@ -714,28 +813,36 @@ chunk_t *chunk_get_prev_nvb(chunk_t *cur, const scope_e scope)
 
 void set_chunk_type_real(chunk_t *pc, c_token_t tt)
 {
-   set_chunk_real(pc, tt, LSETTYP, "set_chunk_type");
+   set_chunk_real(pc, tt, LSETTYP);
 }
 
 
 void set_chunk_parent_real(chunk_t *pc, c_token_t pt)
 {
-   set_chunk_real(pc, pt, LSETPAR, "set_chunk_parent");
+   set_chunk_real(pc, pt, LSETPAR);
 }
 
 
-void chunk_flags_set_real(chunk_t *pc, UINT64 clr_bits, UINT64 set_bits)
+void chunk_flags_set_real(chunk_t *pc, pcf_flags_t clr_bits, pcf_flags_t set_bits)
 {
    if (pc != nullptr)
    {
       LOG_FUNC_ENTRY();
-      UINT64 nflags = (pc->flags & ~clr_bits) | set_bits;
+      auto const nflags = (pc->flags & ~clr_bits) | set_bits;
+
       if (pc->flags != nflags)
       {
-         LOG_FMT(LSETFLG, "set_chunk_flags(%d): %016" PRIx64 "^%016" PRIx64 "=%016" PRIx64 " %zu:%zu '%s' %s:%s",
-                 __LINE__, pc->flags, pc->flags ^ nflags, nflags,
+         LOG_FMT(LSETFLG,
+                 "%s(%d): %016llx^%016llx=%016llx "
+                 "orig_line is %zu, orig_col is %zu, text() '%s', type is %s, ",
+                 __func__, __LINE__,
+                 static_cast<pcf_flags_t::int_t>(pc->flags),
+                 static_cast<pcf_flags_t::int_t>(pc->flags ^ nflags),
+                 static_cast<pcf_flags_t::int_t>(nflags),
                  pc->orig_line, pc->orig_col, pc->text(),
-                 get_token_name(pc->type), get_token_name(pc->parent_type));
+                 get_token_name(pc->type));
+         LOG_FMT(LSETFLG, "parent_type is %s",
+                 get_token_name(pc->parent_type));
          log_func_stack_inline(LSETFLG);
          pc->flags = nflags;
       }
@@ -743,7 +850,7 @@ void chunk_flags_set_real(chunk_t *pc, UINT64 clr_bits, UINT64 set_bits)
 }
 
 
-void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what, const char *str)
+void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what)
 {
    LOG_FUNC_ENTRY();
 
@@ -771,15 +878,25 @@ void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what, const char *st
 
    if (pc != nullptr && *where != token)
    {
-      LOG_FMT(what, "%s(%d): orig_line is %zu, orig_col is %zu, pc->text() '%s'\n",
-              str, __LINE__, pc->orig_line, pc->orig_col, pc->text());
+      LOG_FMT(what, "%s(%d): orig_line is %zu, orig_col is %zu, pc->text() ",
+              __func__, __LINE__, pc->orig_line, pc->orig_col);
+
+      if (*type == CT_NEWLINE)
+      {
+         LOG_FMT(what, "<Newline>\n");
+      }
+      else
+      {
+         LOG_FMT(what, "'%s'\n",
+                 pc->text());
+      }
       LOG_FMT(what, "   pc->type is %s, pc->parent_type is %s => *type is %s, *parent_type is %s",
               get_token_name(pc->type), get_token_name(pc->parent_type),
               get_token_name(*type), get_token_name(*parent_type));
       log_func_stack_inline(what);
       *where = token;
    }
-}
+} // set_chunk_real
 
 
 static chunk_t *chunk_get_ncnlnp(chunk_t *cur, const scope_e scope, const direction_e dir)
@@ -793,24 +910,26 @@ static chunk_t *chunk_get_ncnlnp(chunk_t *cur, const scope_e scope, const direct
 }
 
 
-static chunk_t *chunk_get_ncnlnpnd(chunk_t *cur, const scope_e scope, const direction_e dir)
-{
-   search_t search_function = select_search_fct(dir);
-   chunk_t  *pc             = cur;
-
-   do                                   // loop over the chunk list
-   {
-      pc = search_function(pc, scope);  // in either direction while
-   } while (  pc != nullptr             // the end of the list was not reached yet
-           && !chunk_is_comment_or_newline(pc)
-           && !chunk_is_preproc(pc)
-           && (pc->type == CT_DC_MEMBER));
-   return(pc);
-}
-
-
 static chunk_t *chunk_add(const chunk_t *pc_in, chunk_t *ref, const direction_e pos)
 {
+#ifdef DEBUG
+   if (pc_in->orig_line == 0)
+   {
+      fprintf(stderr, "%s(%d): no line number\n", __func__, __LINE__);
+      log_func_stack_inline(LSETFLG);
+      log_flush(true);
+      exit(EX_SOFTWARE);
+   }
+
+   if (pc_in->orig_col == 0)
+   {
+      fprintf(stderr, "%s(%d): no column number\n", __func__, __LINE__);
+      log_func_stack_inline(LSETFLG);
+      log_flush(true);
+      exit(EX_SOFTWARE);
+   }
+#endif /* DEBUG */
+
    chunk_t *pc = chunk_dup(pc_in);
 
    if (pc != nullptr)
@@ -823,7 +942,76 @@ static chunk_t *chunk_add(const chunk_t *pc_in, chunk_t *ref, const direction_e
       {
          (pos == direction_e::FORWARD) ? g_cl.AddHead(pc) : g_cl.AddTail(pc);
       }
-      chunk_log(pc, "chunk_add");
+      chunk_log(pc, "chunk_add(A):");
    }
    return(pc);
 }
+
+
+chunk_t *chunk_get_next_ssq(chunk_t *cur)
+{
+   while (chunk_is_token(cur, CT_TSQUARE) || chunk_is_token(cur, CT_SQUARE_OPEN))
+   {
+      if (chunk_is_token(cur, CT_SQUARE_OPEN))
+      {
+         cur = chunk_skip_to_match(cur);
+      }
+      cur = chunk_get_next_ncnl(cur);
+   }
+   return(cur);
+}
+
+
+chunk_t *chunk_get_prev_ssq(chunk_t *cur)
+{
+   while (chunk_is_token(cur, CT_TSQUARE) || chunk_is_token(cur, CT_SQUARE_CLOSE))
+   {
+      if (chunk_is_token(cur, CT_SQUARE_CLOSE))
+      {
+         cur = chunk_skip_to_match_rev(cur);
+      }
+      cur = chunk_get_prev_ncnl(cur);
+   }
+   return(cur);
+}
+
+
+//! skip to the final word/type in a :: chain
+static chunk_t *chunk_skip_dc_member(chunk_t *start, scope_e scope, direction_e dir)
+{
+   LOG_FUNC_ENTRY();
+
+   if (start == nullptr)
+   {
+      return(nullptr);
+   }
+   const auto step_fcn = (dir == direction_e::FORWARD)
+                         ? chunk_get_next_ncnl : chunk_get_prev_ncnl;
+
+   chunk_t *pc   = start;
+   chunk_t *next = chunk_is_token(pc, CT_DC_MEMBER) ? pc : step_fcn(pc, scope);
+
+   while (chunk_is_token(next, CT_DC_MEMBER))
+   {
+      pc = step_fcn(next, scope);
+
+      if (pc == nullptr)
+      {
+         return(nullptr);
+      }
+      next = step_fcn(pc, scope);
+   }
+   return(pc);
+}
+
+
+chunk_t *chunk_skip_dc_member(chunk_t *start, scope_e scope)
+{
+   return(chunk_skip_dc_member(start, scope, direction_e::FORWARD));
+}
+
+
+chunk_t *chunk_skip_dc_member_rev(chunk_t *start, scope_e scope)
+{
+   return(chunk_skip_dc_member(start, scope, direction_e::BACKWARD));
+}
