@@ -9,7 +9,9 @@
 #define CHUNK_LIST_H_INCLUDED
 
 #include "uncrustify_types.h"
+// necessary to not sort it
 #include "char_table.h"
+#include "language_tools.h"
 
 
 /*
@@ -40,7 +42,7 @@ enum class scope_e : unsigned int
 };
 
 
-void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what, const char *str);
+void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what);
 
 
 /**
@@ -188,7 +190,7 @@ chunk_t *chunk_get_next_nc(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
 /**
- * Gets the next non-NEWLINE and non-comment chunk
+ * Gets the next non-NEWLINE
  *
  * @param cur    chunk to use as start point
  * @param scope  code region to search in
@@ -197,7 +199,7 @@ chunk_t *chunk_get_next_nnl(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
 /**
- * Gets the next non-NEWLINE and non-comment chunk, non-preprocessor chunk
+ * Gets the next non-NEWLINE and non-comment chunk
  *
  * @param cur    chunk to use as start point
  * @param scope  code region to search in
@@ -214,6 +216,18 @@ chunk_t *chunk_get_next_ncnl(chunk_t *cur, scope_e scope = scope_e::ALL);
 chunk_t *chunk_get_next_ncnlnp(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
+/**
+ * Gets the next non-NEWLINE and non-comment chunk (preprocessor aware).
+ * Unlike chunk_get_next_ncnl, this will also ignore a line continuation if
+ * the starting chunk is in a preprocessor directive, and may return a newline
+ * if the search reaches the end of a preprocessor directive.
+ *
+ * @param cur    chunk to use as start point
+ * @param scope  code region to search in
+ */
+chunk_t *chunk_ppa_get_next_ncnl(chunk_t *cur);
+
+
 /**
  * Gets the next chunk not in or part of balanced square
  * brackets. This handles stacked [] instances to accommodate
@@ -281,6 +295,15 @@ chunk_t *chunk_get_prev_nnl(chunk_t *cur, scope_e scope = scope_e::ALL);
 chunk_t *chunk_get_prev_ncnl(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
+/**
+ * Gets the prev non-NEWLINE and non-comment and non-ignored chunk
+ *
+ * @param cur    chunk to use as start point
+ * @param scope  code region to search in
+ */
+chunk_t *chunk_get_prev_ncnlni(chunk_t *cur, scope_e scope = scope_e::ALL);
+
+
 /**
  * Gets the prev non-NEWLINE and non-comment chunk, non-preprocessor chunk
  *
@@ -290,15 +313,6 @@ chunk_t *chunk_get_prev_ncnl(chunk_t *cur, scope_e scope = scope_e::ALL);
 chunk_t *chunk_get_prev_ncnlnp(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
-/**
- * Gets the prev non-NEWLINE and non-comment chunk, non-preprocessor chunk, non-DC_MEMBER chunk
- *
- * @param cur    chunk to use as start point
- * @param scope  code region to search in
- */
-chunk_t *chunk_get_prev_ncnlnpnd(chunk_t *cur, scope_e scope = scope_e::ALL);
-
-
 /**
  * Grabs the next chunk of the given type at the level.
  *
@@ -381,6 +395,28 @@ chunk_t *chunk_get_next_nvb(chunk_t *cur, const scope_e scope = scope_e::ALL);
 chunk_t *chunk_get_prev_nvb(chunk_t *cur, const scope_e scope = scope_e::ALL);
 
 
+/**
+ * Gets the next chunk not in or part of balanced square
+ * brackets.This handles stacked[] instances to accommodate
+ * multi - dimensional array declarations
+ *
+ * @param  cur    chunk to use as start point
+ *
+ * @return nullptr or the next chunk not in or part of square brackets
+ */
+chunk_t *chunk_get_next_ssq(chunk_t *cur);
+
+/**
+ * Gets the prev chunk not in or part of balanced square
+ * brackets.This handles stacked[] instances to accommodate
+ * multi - dimensional array declarations
+ *
+ * @param  cur    chunk to use as start point
+ *
+ * @return nullptr or the prev chunk not in or part of square brackets
+ */
+chunk_t *chunk_get_prev_ssq(chunk_t *cur);
+
 /**
  * @brief reverse search a chunk of a given category in a chunk list
  *
@@ -404,6 +440,16 @@ chunk_t *chunk_search_prev_cat(chunk_t *pc, const c_token_t cat);
  */
 chunk_t *chunk_search_next_cat(chunk_t *pc, const c_token_t cat);
 
+/**
+ * @brief checks wether two chunks are in same line
+ *
+ * @param  start
+ * @param  end
+ *
+ * @return true if there is no newline between start and end chunks
+ */
+bool are_chunks_in_same_line(chunk_t *start, chunk_t *end);
+
 /*
  * TODO: better move the function implementations to the source file.
  * No need to make the implementation public.
@@ -415,7 +461,7 @@ chunk_t *chunk_search_next_cat(chunk_t *pc, const c_token_t cat);
  * The compiler should know how to optimize the code itself.
  * To clarify do a profiling run with and without inline
  */
-static_inline bool is_expected_type_and_level(chunk_t *pc, c_token_t type, int level)
+static inline bool is_expected_type_and_level(chunk_t *pc, c_token_t type, int level)
 {
    // we don't care about the level (if it is negative) or it is as expected
    // and the type is as expected
@@ -424,7 +470,7 @@ static_inline bool is_expected_type_and_level(chunk_t *pc, c_token_t type, int l
 }
 
 
-static_inline bool is_expected_string_and_level(chunk_t *pc, const char *str, int level, size_t len)
+static inline bool is_expected_string_and_level(chunk_t *pc, const char *str, int level, size_t len)
 {
    // we don't care about the level (if it is negative) or it is as expected
    return(  (level < 0 || pc->level == static_cast<size_t>(level))
@@ -433,6 +479,18 @@ static_inline bool is_expected_string_and_level(chunk_t *pc, const char *str, in
 }
 
 
+static inline bool chunk_is_token(const chunk_t *pc, c_token_t c_token)
+{
+   return(pc != nullptr && pc->type == c_token);
+}
+
+
+static inline bool chunk_is_not_token(const chunk_t *pc, c_token_t c_token)
+{
+   return(pc != nullptr && pc->type != c_token);
+}
+
+
 /**
  * Skips to the closing match for the current paren/brace/square.
  *
@@ -441,17 +499,16 @@ static_inline bool is_expected_string_and_level(chunk_t *pc, const char *str, in
  *
  * @return nullptr or the matching paren/brace/square
  */
-static_inline chunk_t *chunk_skip_to_match(chunk_t *cur, scope_e scope = scope_e::ALL)
+static inline chunk_t *chunk_skip_to_match(chunk_t *cur, scope_e scope = scope_e::ALL)
 {
-   if (  cur
-      && (  cur->type == CT_PAREN_OPEN
-         || cur->type == CT_SPAREN_OPEN
-         || cur->type == CT_FPAREN_OPEN
-         || cur->type == CT_TPAREN_OPEN
-         || cur->type == CT_BRACE_OPEN
-         || cur->type == CT_VBRACE_OPEN
-         || cur->type == CT_ANGLE_OPEN
-         || cur->type == CT_SQUARE_OPEN))
+   if (  chunk_is_token(cur, CT_PAREN_OPEN)
+      || chunk_is_token(cur, CT_SPAREN_OPEN)
+      || chunk_is_token(cur, CT_FPAREN_OPEN)
+      || chunk_is_token(cur, CT_TPAREN_OPEN)
+      || chunk_is_token(cur, CT_BRACE_OPEN)
+      || chunk_is_token(cur, CT_VBRACE_OPEN)
+      || chunk_is_token(cur, CT_ANGLE_OPEN)
+      || chunk_is_token(cur, CT_SQUARE_OPEN))
    {
       return(chunk_get_next_type(cur, (c_token_t)(cur->type + 1), cur->level, scope));
    }
@@ -459,17 +516,16 @@ static_inline chunk_t *chunk_skip_to_match(chunk_t *cur, scope_e scope = scope_e
 }
 
 
-static_inline chunk_t *chunk_skip_to_match_rev(chunk_t *cur, scope_e scope = scope_e::ALL)
+static inline chunk_t *chunk_skip_to_match_rev(chunk_t *cur, scope_e scope = scope_e::ALL)
 {
-   if (  cur
-      && (  cur->type == CT_PAREN_CLOSE
-         || cur->type == CT_SPAREN_CLOSE
-         || cur->type == CT_FPAREN_CLOSE
-         || cur->type == CT_TPAREN_CLOSE
-         || cur->type == CT_BRACE_CLOSE
-         || cur->type == CT_VBRACE_CLOSE
-         || cur->type == CT_ANGLE_CLOSE
-         || cur->type == CT_SQUARE_CLOSE))
+   if (  chunk_is_token(cur, CT_PAREN_CLOSE)
+      || chunk_is_token(cur, CT_SPAREN_CLOSE)
+      || chunk_is_token(cur, CT_FPAREN_CLOSE)
+      || chunk_is_token(cur, CT_TPAREN_CLOSE)
+      || chunk_is_token(cur, CT_BRACE_CLOSE)
+      || chunk_is_token(cur, CT_VBRACE_CLOSE)
+      || chunk_is_token(cur, CT_ANGLE_CLOSE)
+      || chunk_is_token(cur, CT_SQUARE_CLOSE))
    {
       return(chunk_get_prev_type(cur, (c_token_t)(cur->type - 1), cur->level, scope));
    }
@@ -477,6 +533,11 @@ static_inline chunk_t *chunk_skip_to_match_rev(chunk_t *cur, scope_e scope = sco
 }
 
 
+//! skip to the final word/type in a :: chain
+chunk_t *chunk_skip_dc_member(chunk_t *start, scope_e scope = scope_e::ALL);
+chunk_t *chunk_skip_dc_member_rev(chunk_t *start, scope_e scope = scope_e::ALL);
+
+
 /**
  * checks if a chunk is valid and is a comment
  *
@@ -486,33 +547,29 @@ static_inline chunk_t *chunk_skip_to_match_rev(chunk_t *cur, scope_e scope = sco
  * - C comment
  * - C++ comment
  */
-static_inline bool chunk_is_comment(chunk_t *pc)
+static inline bool chunk_is_comment(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (  pc->type == CT_COMMENT
-            || pc->type == CT_COMMENT_MULTI
-            || pc->type == CT_COMMENT_CPP));
+   return(  chunk_is_token(pc, CT_COMMENT)
+         || chunk_is_token(pc, CT_COMMENT_MULTI)
+         || chunk_is_token(pc, CT_COMMENT_CPP));
 }
 
 
-static_inline bool chunk_is_single_line_comment(chunk_t *pc)
+static inline bool chunk_is_single_line_comment(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_COMMENT || pc->type == CT_COMMENT_CPP));
+   return(chunk_is_token(pc, CT_COMMENT) || chunk_is_token(pc, CT_COMMENT_CPP));
 }
 
 
-static_inline bool chunk_is_newline(chunk_t *pc)
+static inline bool chunk_is_newline(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_NEWLINE || pc->type == CT_NL_CONT));
+   return(chunk_is_token(pc, CT_NEWLINE) || chunk_is_token(pc, CT_NL_CONT));
 }
 
 
-static_inline bool chunk_is_semicolon(chunk_t *pc)
+static inline bool chunk_is_semicolon(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_SEMICOLON || pc->type == CT_VSEMICOLON));
+   return(chunk_is_token(pc, CT_SEMICOLON) || chunk_is_token(pc, CT_VSEMICOLON));
 }
 
 
@@ -523,43 +580,49 @@ static_inline bool chunk_is_semicolon(chunk_t *pc)
  *
  * @todo rename function: blank is a space not an empty string
  */
-static_inline bool chunk_is_blank(chunk_t *pc)
+static inline bool chunk_is_blank(chunk_t *pc)
 {
-   return(pc != NULL && (pc->len() == 0));
+   return(pc != nullptr && (pc->len() == 0));
 }
 
 
 //! checks if a chunk is valid and either a comment or newline
-static_inline bool chunk_is_comment_or_newline(chunk_t *pc)
+static inline bool chunk_is_comment_or_newline(chunk_t *pc)
 {
    return(chunk_is_comment(pc) || chunk_is_newline(pc));
 }
 
 
-static_inline bool chunk_is_balanced_square(chunk_t *pc)
+//! checks if a chunk is valid and either a comment or newline or ignored
+static inline bool chunk_is_comment_or_newline_or_ignored(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (  pc->type == CT_SQUARE_OPEN
-            || pc->type == CT_TSQUARE
-            || pc->type == CT_SQUARE_CLOSE));
+   return(chunk_is_comment(pc) || chunk_is_newline(pc) || chunk_is_token(pc, CT_IGNORED));
 }
 
 
-static_inline bool chunk_is_preproc(chunk_t *pc)
+static inline bool chunk_is_balanced_square(chunk_t *pc)
 {
-   return(pc != NULL && (pc->flags & PCF_IN_PREPROC));
+   return(  chunk_is_token(pc, CT_SQUARE_OPEN)
+         || chunk_is_token(pc, CT_TSQUARE)
+         || chunk_is_token(pc, CT_SQUARE_CLOSE));
 }
 
 
-static_inline bool chunk_is_comment_or_newline_in_preproc(chunk_t *pc)
+static inline bool chunk_is_preproc(chunk_t *pc)
 {
-   return(  pc != NULL
+   return(pc != nullptr && pc->flags.test(PCF_IN_PREPROC));
+}
+
+
+static inline bool chunk_is_comment_or_newline_in_preproc(chunk_t *pc)
+{
+   return(  pc != nullptr
          && chunk_is_preproc(pc)
          && (chunk_is_comment(pc) || chunk_is_newline(pc)));
 }
 
 
-static_inline bool chunk_is_comment_newline_or_preproc(chunk_t *pc)
+static inline bool chunk_is_comment_newline_or_preproc(chunk_t *pc)
 {
    return(  chunk_is_comment(pc)
          || chunk_is_newline(pc)
@@ -567,13 +630,13 @@ static_inline bool chunk_is_comment_newline_or_preproc(chunk_t *pc)
 }
 
 
-static_inline bool chunk_is_comment_newline_or_blank(chunk_t *pc)
+static inline bool chunk_is_comment_newline_or_blank(chunk_t *pc)
 {
    return(chunk_is_comment_or_newline(pc) || chunk_is_blank(pc));
 }
 
 
-static_inline bool chunk_is_Doxygen_comment(chunk_t *pc)
+static inline bool chunk_is_Doxygen_comment(chunk_t *pc)
 {
    if (!chunk_is_comment(pc))
    {
@@ -582,6 +645,7 @@ static_inline bool chunk_is_Doxygen_comment(chunk_t *pc)
    // check the third character
    const char   *sComment = pc->text();
    const size_t len       = strlen(sComment);
+
    if (len < 3)
    {
       return(false);
@@ -592,29 +656,22 @@ static_inline bool chunk_is_Doxygen_comment(chunk_t *pc)
 }
 
 
-static_inline bool chunk_is_type(chunk_t *pc)
+static inline bool chunk_is_type(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (  pc->type == CT_TYPE
-            || pc->type == CT_PTR_TYPE
-            || pc->type == CT_BYREF
-            || pc->type == CT_DC_MEMBER
-            || pc->type == CT_QUALIFIER
-            || pc->type == CT_STRUCT
-            || pc->type == CT_ENUM
-            || pc->type == CT_UNION));
+   return(  chunk_is_token(pc, CT_TYPE)
+         || chunk_is_token(pc, CT_PTR_TYPE)
+         || chunk_is_token(pc, CT_BYREF)
+         || chunk_is_token(pc, CT_DC_MEMBER)
+         || chunk_is_token(pc, CT_QUALIFIER)
+         || chunk_is_token(pc, CT_STRUCT)
+         || chunk_is_token(pc, CT_ENUM)
+         || chunk_is_token(pc, CT_UNION));
 }
 
 
-static_inline bool chunk_is_token(chunk_t *pc, c_token_t c_token)
+static inline bool chunk_is_str(chunk_t *pc, const char *str, size_t len)
 {
-   return(pc != NULL && pc->type == c_token);
-}
-
-
-static_inline bool chunk_is_str(chunk_t *pc, const char *str, size_t len)
-{
-   return(  pc != NULL                            // valid pc pointer
+   return(  pc != nullptr                         // valid pc pointer
          && (pc->len() == len)                    // token size equals size parameter
          && (memcmp(pc->text(), str, len) == 0)); // token name is the same as str parameter
 
@@ -625,70 +682,73 @@ static_inline bool chunk_is_str(chunk_t *pc, const char *str, size_t len)
 }
 
 
-static_inline bool chunk_is_str_case(chunk_t *pc, const char *str, size_t len)
+static inline bool chunk_is_str_case(chunk_t *pc, const char *str, size_t len)
 {
-   return(  pc != NULL
+   return(  pc != nullptr
          && (pc->len() == len)
          && (strncasecmp(pc->text(), str, len) == 0));
 }
 
 
-static_inline bool chunk_is_word(chunk_t *pc)
+static inline bool chunk_is_word(chunk_t *pc)
 {
-   return(  pc != NULL
+   return(  pc != nullptr
          && (pc->len() >= 1)
          && CharTable::IsKw1(pc->str[0]));
 }
 
 
-static_inline bool chunk_is_star(chunk_t *pc)
+static inline bool chunk_is_star(chunk_t *pc)
 {
-   return(  pc != NULL
+   return(  pc != nullptr
          && (pc->len() == 1)
          && (pc->str[0] == '*')
          && pc->type != CT_OPERATOR_VAL);
 }
 
 
-static_inline bool chunk_is_addr(chunk_t *pc)
+static inline bool chunk_is_nullable(chunk_t *pc)
 {
-   if (  pc != NULL
-      && (  pc->type == CT_BYREF
-         || (  (pc->len() == 1)
-            && (pc->str[0] == '&')
-            && pc->type != CT_OPERATOR_VAL)))
+   return(language_is_set(LANG_CS) && (pc != nullptr) && (pc->len() == 1) && (pc->str[0] == '?'));
+}
+
+
+static inline bool chunk_is_addr(chunk_t *pc)
+{
+   if (  chunk_is_token(pc, CT_BYREF)
+      || (  (pc->len() == 1)
+         && (pc->str[0] == '&')
+         && pc->type != CT_OPERATOR_VAL))
    {
       chunk_t *prev = chunk_get_prev(pc);
 
-      if (  (pc->flags & PCF_IN_TEMPLATE)
-         && (  prev != NULL
-            && (prev->type == CT_COMMA || prev->type == CT_ANGLE_OPEN)))
+      if (  pc->flags.test(PCF_IN_TEMPLATE)
+         && (chunk_is_token(prev, CT_COMMA) || chunk_is_token(prev, CT_ANGLE_OPEN)))
       {
          return(false);
       }
-
       return(true);
    }
-
    return(false);
 }
 
 
-static_inline bool chunk_is_msref(chunk_t *pc) // ms compilers for C++/CLI and WinRT use '^' instead of '*' for marking up reference types vs pointer types
+static inline bool chunk_is_msref(chunk_t *pc) // ms compilers for C++/CLI and WinRT use '^' instead of '*' for marking up reference types vs pointer types
 {
-   return(  (cpd.lang_flags & LANG_CPP)
-         && (  pc != NULL
+   return(  language_is_set(LANG_CPP)
+         && (  pc != nullptr
             && (pc->len() == 1)
             && (pc->str[0] == '^')
             && pc->type != CT_OPERATOR_VAL));
 }
 
 
-static_inline bool chunk_is_ptr_operator(chunk_t *pc)
+static inline bool chunk_is_ptr_operator(chunk_t *pc)
 {
-   return(  chunk_is_star(pc)
-         || chunk_is_addr(pc)
-         || chunk_is_msref(pc));
+   return(  (  chunk_is_star(pc)
+            || chunk_is_addr(pc)
+            || chunk_is_msref(pc))
+         || chunk_is_nullable(pc));
 }
 
 
@@ -696,44 +756,39 @@ static_inline bool chunk_is_ptr_operator(chunk_t *pc)
 bool chunk_is_newline_between(chunk_t *start, chunk_t *end);
 
 
-static_inline bool chunk_is_closing_brace(chunk_t *pc)
+static inline bool chunk_is_closing_brace(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_BRACE_CLOSE || pc->type == CT_VBRACE_CLOSE));
+   return(chunk_is_token(pc, CT_BRACE_CLOSE) || chunk_is_token(pc, CT_VBRACE_CLOSE));
 }
 
 
-static_inline bool chunk_is_opening_brace(chunk_t *pc)
+static inline bool chunk_is_opening_brace(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_BRACE_OPEN || pc->type == CT_VBRACE_OPEN));
+   return(chunk_is_token(pc, CT_BRACE_OPEN) || chunk_is_token(pc, CT_VBRACE_OPEN));
 }
 
 
-static_inline bool chunk_is_vbrace(chunk_t *pc)
+static inline bool chunk_is_vbrace(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (pc->type == CT_VBRACE_CLOSE || pc->type == CT_VBRACE_OPEN));
+   return(chunk_is_token(pc, CT_VBRACE_CLOSE) || chunk_is_token(pc, CT_VBRACE_OPEN));
 }
 
 
-static_inline bool chunk_is_paren_open(chunk_t *pc)
+static inline bool chunk_is_paren_open(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (  pc->type == CT_PAREN_OPEN
-            || pc->type == CT_SPAREN_OPEN
-            || pc->type == CT_TPAREN_OPEN
-            || pc->type == CT_FPAREN_OPEN));
+   return(  chunk_is_token(pc, CT_PAREN_OPEN)
+         || chunk_is_token(pc, CT_SPAREN_OPEN)
+         || chunk_is_token(pc, CT_TPAREN_OPEN)
+         || chunk_is_token(pc, CT_FPAREN_OPEN));
 }
 
 
-static_inline bool chunk_is_paren_close(chunk_t *pc)
+static inline bool chunk_is_paren_close(chunk_t *pc)
 {
-   return(  pc != NULL
-         && (  pc->type == CT_PAREN_CLOSE
-            || pc->type == CT_SPAREN_CLOSE
-            || pc->type == CT_TPAREN_CLOSE
-            || pc->type == CT_FPAREN_CLOSE));
+   return(  chunk_is_token(pc, CT_PAREN_CLOSE)
+         || chunk_is_token(pc, CT_SPAREN_CLOSE)
+         || chunk_is_token(pc, CT_TPAREN_CLOSE)
+         || chunk_is_token(pc, CT_FPAREN_CLOSE));
 }
 
 
@@ -741,10 +796,10 @@ static_inline bool chunk_is_paren_close(chunk_t *pc)
  * Returns true if either chunk is null or both have the same preproc flags.
  * If this is true, you can remove a newline/nl_cont between the two.
  */
-static_inline bool chunk_same_preproc(chunk_t *pc1, chunk_t *pc2)
+static inline bool chunk_same_preproc(chunk_t *pc1, chunk_t *pc2)
 {
-   return(  pc1 == NULL
-         || pc2 == NULL
+   return(  pc1 == nullptr
+         || pc2 == nullptr
          || ((pc1->flags & PCF_IN_PREPROC) == (pc2->flags & PCF_IN_PREPROC)));
 }
 
@@ -754,11 +809,11 @@ static_inline bool chunk_same_preproc(chunk_t *pc1, chunk_t *pc2)
  * The prev and next chunks must have the same PCF_IN_PREPROC flag AND
  * the newline can't be after a C++ comment.
  */
-static_inline bool chunk_safe_to_del_nl(chunk_t *nl)
+static inline bool chunk_safe_to_del_nl(chunk_t *nl)
 {
    chunk_t *tmp = chunk_get_prev(nl);
 
-   if (tmp != NULL && tmp->type == CT_COMMENT_CPP)
+   if (chunk_is_token(tmp, CT_COMMENT_CPP))
    {
       return(false);
    }
@@ -772,23 +827,25 @@ static_inline bool chunk_safe_to_del_nl(chunk_t *nl)
  *
  * @return true  - the chunk is the opening parentheses of a for in loop
  */
-static_inline bool chunk_is_forin(chunk_t *pc)
+static inline bool chunk_is_forin(chunk_t *pc)
 {
-   if (  (cpd.lang_flags & LANG_OC)
-      && pc
-      && pc->type == CT_SPAREN_OPEN)
+   if (  language_is_set(LANG_OC)
+      && chunk_is_token(pc, CT_SPAREN_OPEN))
    {
       chunk_t *prev = chunk_get_prev_ncnl(pc);
-      if (prev->type == CT_FOR)
+
+      if (chunk_is_token(prev, CT_FOR))
       {
          chunk_t *next = pc;
-         while (  next
+
+         while (  next != nullptr
                && next->type != CT_SPAREN_CLOSE
                && next->type != CT_IN)
          {
             next = chunk_get_next_ncnl(next);
          }
-         if (next->type == CT_IN)
+
+         if (chunk_is_token(next, CT_IN))
          {
             return(true);
          }
@@ -815,7 +872,7 @@ void set_chunk_parent_real(chunk_t *pc, c_token_t tt);
 } while (false)
 
 
-void chunk_flags_set_real(chunk_t *pc, UINT64 clr_bits, UINT64 set_bits);
+void chunk_flags_set_real(chunk_t *pc, pcf_flags_t clr_bits, pcf_flags_t set_bits);
 
 
 #define chunk_flags_upd(pc, cc, ss)    do {   \
@@ -825,12 +882,12 @@ void chunk_flags_set_real(chunk_t *pc, UINT64 clr_bits, UINT64 set_bits);
 
 #define chunk_flags_set(pc, ss)        do { \
       LOG_FUNC_CALL();                      \
-      chunk_flags_set_real((pc), 0, (ss));  \
+      chunk_flags_set_real((pc), {}, (ss)); \
 } while (false)
 
 #define chunk_flags_clr(pc, cc)        do { \
       LOG_FUNC_CALL();                      \
-      chunk_flags_set_real((pc), (cc), 0);  \
+      chunk_flags_set_real((pc), (cc), {}); \
 } while (false)
 
 
